# -*- coding: utf-8 -*-
""" FHCoreLibs > ADMIN > Inline > Generic.py by Frankhood
    @author: Frankhood Business Solutions
"""
from django.contrib.contenttypes.generic import GenericStackedInline
from django.forms.models import BaseModelFormSet

class LocaleGenericInlineFormSet(BaseModelFormSet):
    """
    A formset for generic inline objects to a parent.
    Is very similar to GenericInlineFormSet but adds Language informations and filtering
    """
    language_field='lang'
    
    def __init__(self, data=None, files=None, instance=None, save_as_new=None,
                 prefix=None, queryset=None):
        # Avoid a circular import.
        from django.contrib.contenttypes.models import ContentType
        opts = self.model._meta
        self.instance = instance
        self.rel_name = '-'.join((
            opts.app_label, opts.object_name.lower(),
            self.ct_field.name, self.ct_fk_field.name,
        ))
        if self.instance is None or self.instance.pk is None:
            qs = self.model._default_manager.none()
        else:
            if queryset is None:
                queryset = self.model._default_manager
            _language = None
            if hasattr(self.instance,'title_cache') and self.instance.title_cache:
                #title cache is the only information about current language... i don't have request here to take it from querystring
                _language = self.instance.title_cache.keys()[0]
            elif getattr(self.instance,self.language_field):
                _language= getattr(self.instance,self.language_field)
                
            qs = queryset.filter(**{
                self.ct_field.name: ContentType.objects.get_for_model(self.instance),
                self.ct_fk_field.name: self.instance.pk,
                self.language_field: _language
            })
            
            if not _language:
                qs=queryset.none()
                
        super(LocaleGenericInlineFormSet, self).__init__(
            queryset=qs, data=data, files=files,
            prefix=prefix
        )


class LocaleGenericStackedInline(GenericStackedInline):
    formset = LocaleGenericInlineFormSet
