# -*- coding: utf-8 -*-
"""  fhcore.admin.translation_admin.py   
    @author: Frankhood Business Solutions
    @date : 22/mar/2013
"""
import os
from django.conf import settings
from django.utils.translation import ungettext, ugettext as _
from django.contrib import admin
from modeltranslation.translator import translator
from modeltranslation.admin import TranslationAdmin
from fhcore.settings import PUBLISHED_CODE, DRAFT_CODE

def language_status(translation_status, lang):
    status = None
    try:
        fields = translation_status['localized'][lang]
        localizable_fields = translation_status['localizable_fields']
        empty_fields = translation_status['localized_fields']['missing']
    except Exception, e:
        pass
    finally:
        if len(fields)==len(localizable_fields):
            status = 'complete'
        #else:
        #    if len(fields)== (len(localizable_fields)-len(empty_fields)):
        #        status = 'complete'
        if not status:
            if len(fields)==0:
                status = 'missing'
            else:
                status = 'partial'
    return status

def instance_translation_status(instance):
    status_dict = {}
    instance_type = type(instance)
    languages = []
    try:
        languages = [lang[0] for lang in settings.LANGUAGES]
    except Exception, e:
        pass
    finally:
        if instance_type in translator.get_registered_models():
            tf = translator.get_options_for_model(instance_type)
            localizable_fields = tf.fields.keys()
            se = {}
            se['languages'] = languages
            se['localizable_fields'] = localizable_fields
            se['localized'] = {}
            for l in languages:
                se['localized'][l] = []
            se['status'] = 'complete' # 'complete', 'partial', missing
            se['localized_fields'] = {'complete': [], 'partial': [], 'missing': []}
            for f in tf.fields.keys():
                subfields = tf.fields[f]
                n_el = len(subfields)
                count = 0
                for sf in subfields:
                    vl = getattr(instance,sf.name,None)
                    count+=(vl!=None)
                    if vl:
                        se['localized'][sf.language].append(f)
                if (count==0):
                    se['localized_fields']['missing'].append(f)
                else:
                    if (count==n_el):
                        se['localized_fields']['complete'].append(f)
                    else:
                        se['localized_fields']['partial'].append(f)
    status_dict = se
    return status_dict


class ModelTranslationAdminMixin(TranslationAdmin): 
    translation_field = 'translation_status'
    
    def translation_status(self, obj):
        ts = instance_translation_status(obj)
        languages = ts["languages"]
        tag = "<div class='fh-translation-group'>"
        for l in languages:
            status = language_status(ts,l)
            ldiv = "<div class='fh-translation-status fh-translation-%s-color'>\
                        <span class='fh-translation-status-lang'>%s</span>\
                        <span class='fh-translation-status-desc'>%s</span>\
                    </div>" % (status, l.upper(), status)
            tag+=ldiv
        tag += '</div>'
        return tag
    translation_status.short_description = _("translation status")
    translation_status.allow_tags = True



    def __init__(self, *args, **kwargs):
        super(ModelTranslationAdminMixin, self).__init__(*args, **kwargs)
        if self.translation_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.translation_field]
        
    def _media(self):
        js = (
            'modeltranslation/js/force_jquery.js',
            'https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/jquery-ui.min.js',
            'modeltranslation/js/tabbed_translation_fields.js',
        )
        css = {
            'screen': (
                'modeltranslation/css/tabbed_translation_fields.css',
                'fhcore/modeltranslation/css/style.css'
            ),
        }
        currmedia = super(ModelTranslationAdminMixin, self).media
        #currmedia = super(ModelTranslationAdminMixin, self)._media()
        old_js=list(currmedia._js)
        old_js.extend(js)
        currmedia._js=tuple(old_js)
        old_css=list(currmedia._css.get('screen',[]))
        old_css.extend(css['screen'])
        currmedia._css['screen']=old_css
        return currmedia
    media = property(_media)


class MultiLanguageModelMixin(admin.ModelAdmin):
    translation_field = 'translation_status'
    
    def translation_status(self, obj):
        
        languages = settings.LANGUAGES
        tag = "<div class='fh-translation-group'>"
        for lang in languages:
            local_object = obj.get_localized_object(lang[0])
            if local_object and local_object.status == PUBLISHED_CODE:
                status = 'complete'
            elif local_object and local_object.status == DRAFT_CODE:
                status = 'not-published'
            else:
                status = 'missing'
            ldiv = "<div class='fh-translation-status fh-translation-%s-color'>\
                        <span class='fh-translation-status-lang'>%s</span>\
                        <span class='fh-translation-status-desc'>%s</span>\
                    </div>" % (status, lang[0].upper(), status)
            tag+=ldiv
        tag += '</div>'
        return tag
    translation_status.short_description = _("translation status")
    translation_status.allow_tags = True


    
    def __init__(self, *args, **kwargs):
        super(MultiLanguageModelMixin, self).__init__(*args, **kwargs)
        if self.translation_field not in self.list_display:
            self.list_display = list(self.list_display) + [self.translation_field]
        
    def _media(self):
        js = (
            'modeltranslation/js/force_jquery.js',
            'https://ajax.googleapis.com/ajax/libs/jqueryui/1.8.2/jquery-ui.min.js',
            'modeltranslation/js/tabbed_translation_fields.js',
        )
        css = {
            'screen': (
                'modeltranslation/css/tabbed_translation_fields.css',
                'fhcore/modeltranslation/css/style.css'
            ),
        }
        currmedia = super(MultiLanguageModelMixin, self).media
        #currmedia = super(ModelTranslationAdminMixin, self)._media()
        old_js=list(currmedia._js)
        old_js.extend(js)
        currmedia._js=tuple(old_js)
        old_css=list(currmedia._css.get('screen',[]))
        old_css.extend(css['screen'])
        currmedia._css['screen']=old_css
        return currmedia
    media = property(_media)
