# -*- coding: utf-8 -*-
"""
Lipton Vota La Spiaggia

Copyright (c) 2013 Frank hood Business solutions srl for IMille S.R.L.
http://www.imille.it

Dual licensed under MIT and GPL3.
http://www.opensource.org/licenses/mit-license.php
http://www.opensource.org/licenses/GPL-3.0

This code and information is provided "as is" without warranty of any
kind, either expressed or implied, including but not limited to the
implied warranties of merchantability and/or fitness for a particular
purpose.

@author Frank Hood Business Solutions srl
@email info(at)frankhood(dot)it
"""
import time

from django import template
from django.conf import settings
from django.template.base import VariableDoesNotExist
from django.template.defaulttags import URLNode
from django.templatetags.future import url as original_url


register = template.Library()

class IframePatchedURLNode(URLNode):
    
    # TODO: personalizzare ssid (settings?)
    
    def render(self, context):
        """
        The render method will return the current domain and path.
        It will try to get the path by calling the view_name variable.
        If that does not work, it will look up the view_name.

        """
        path = super(IframePatchedURLNode, self).render(context)
        request = context.get('request', None)
        if request and getattr(request.session, 'iframe_patched', False):
            if not path.find('?') == -1:
                path = "%s&ssid=%s&_t=%s" % (path, request.session.session_key, time.time())
            else: 
                path = "%s?ssid=%s&_t=%s" % (path, request.session.session_key, time.time())
        else:
            if not path.find('?') == -1:
                path = "%s&_t=%s" % (path, time.time())
            else: 
                path = "%s?_t=%s" % (path, time.time())

        if self.asvar:
            context[self.asvar]= path
            return ''  
        else:  
            return path

@register.tag(name="url")
def patched_url(parser, token, node_cls=IframePatchedURLNode):
    """Just like {% url %} but patch ssid if in iframe """
    node_instance = original_url(parser, token)

    return node_cls(
            view_name=node_instance.view_name,
            args=node_instance.args,
            kwargs=node_instance.kwargs,
            asvar=node_instance.asvar,
        )
