# -*- coding: utf-8 -*-
""" FH CORE LIBS
    @author: Frank Hood Business Solutions Srl
    @copyright: Frank Hood Business Solutions Srl
"""

import cgi
import hmac
import time
import base64
import hashlib
import urlparse
import urllib

from django.conf import settings
from django.contrib import messages
from django.http import HttpResponse
from django.core.exceptions import ImproperlyConfigured
from django.template.loader import get_template
from django.template import Context
from django.views.decorators.csrf import csrf_exempt
from django.views.decorators.cache import never_cache
from django.views.generic import View, TemplateView, ListView, DetailView
from django.views.generic.base import TemplateResponse, TemplateResponseMixin
from django.contrib.sites.models import Site
from django.utils.decorators import method_decorator
from django.utils.encoding import smart_str
from django.utils.translation import ugettext as _u

try:
    from fbconsole import Client as FBClient
except ImportError:
    raise ImproperlyConfigured("FH Facebook Views require the 'fbconsole' cots module")

try:
    # Django <= 1.6 backwards compatibility
    from django.utils import simplejson as json
except ImportError:
    # Django >= 1.7
    import json

#from fhcore.utils.lazy import LazyReverseUrl
from fhcore.apps.facebook.views.exceptions import ActionNotAllowed, NoUserException
from fhcore.views.base import JSONResponseMixin, HeadResponseMixin

import logging
logger = logging.getLogger('project')

try:
    # mandatory
    FB_APP_ID = getattr(settings, 'FB_APP_DATA', {}).get('app_id', '')
    FB_APP_SECRET = getattr(settings, 'FB_APP_DATA', {}).get('app_secret', '')
    FB_APP_SCOPE = () #('email', 'user_birthday', 'user_hometown', 'publish_actions', )

    # non mandatory
    FB_APP_NAME = getattr(settings, 'FB_APP_DATA', {}).get('app_name', '')
    FB_APP_URL = getattr(settings, 'FB_APP_DATA', {}).get('app_url', '')
    FB_CANVAS_URL = getattr(settings, 'FB_APP_DATA', {}).get('canvas_url', '')
    FB_TAB_URL = getattr(settings, 'FB_APP_DATA', {}).get('tab_url', '')
    FB_OAUTH_TEMPLATE = getattr(settings, 'FB_APP_DATA', {}).get('oauth_template', 'facebook/oauth.html')
    FB_APP_PATTERN = 'http://apps.facebook.com/%s/'
    FB_DENIED_URL = "http://www.facebook.com/"
except KeyError, ex:
    msg = u'facebook config error. Please check "FB_APP_DATA" on your settings'
    raise ImproperlyConfigured(msg)

##
# BASE VIEWS
## 



class FBAppViewMixin(object):
    
    fb_app_id = FB_APP_ID
    fb_app_name = FB_APP_NAME
    fb_app_secret = FB_APP_SECRET
    fb_app_url = FB_APP_URL
    fb_tab_url = FB_TAB_URL
    fb_canvas_url = FB_CANVAS_URL
    oauth_dialog_url = 'https://www.facebook.com/dialog/oauth'
    app_scope = FB_APP_SCOPE
    redirect_page = None
    redirect_uri = FB_TAB_URL

    
    """
    WORKFLOW:
    dispatch
        > load_facebook_data
        @==> crea self.facebook_data, self.app_data, self.fb_client(null=True)
            > parse_signed_request
                > base64_url_decode
            > parse_app_data
                > base64_url_decode
        > prehandle_response
        @==> si occupa della Redirect in caso ci sia una condizione 
            > prefetch_data
                > get_userdata_from_facebook
                @==> crea self.fb_user o self.player
                    > get_facebook_user
                    > get_facebook_player
            > get_login_template
                > get_login_url
                    > get_redirect_uri
                        > get_protocol
                        > get_redirect_page
    ? encode_app_data
        > parse_app_data
        
    
    """
    
    @staticmethod
    def base64_url_decode(data):
        """ utility method primarily used for decoding a signed_request
            @return str serialized JSON object.
        """
        data = data.encode(u'ascii')
        data += '=' * (4 - (len(data) % 4))
        return base64.urlsafe_b64decode(data)
    
    
    def parse_signed_request(self, signed_request):
        """ resolve a Facebook signed request 
            @return dict
        """
        facebook_data = {}
        try:
            sig, payload = signed_request.split('.', 1)
            sig = self.base64_url_decode(sig)
            data = json.loads(self.base64_url_decode(payload))
            expected_sig = hmac.new(
                self.fb_app_secret,
                msg=payload,
                digestmod=hashlib.sha256).digest()
            # allow the signed_request to function for up to 1 day
            facebook_data['signed_request'] = data
            if sig == expected_sig and data['issued_at'] > (time.time() - 86400):
                facebook_data['user_id'] = data.get('user_id')
                facebook_data['access_token'] = data.get('oauth_token')
            else:
                try:
                    facebook_data = json.loads(data.user)
                except Exception, ex:
                    logger.error(ex)
                    pass
        except ValueError, ex:
            # ignore if can't split on dot
            logger.excpetion("parse_signed_request raise")
        return facebook_data
    
    
    def parse_app_data(self, request, facebook_data=None):
        """ search the 'app_data' parameter either in the FB signed request or
            in the request POST/GET dict and parse it.
            @note: we suppose that 'app_data' contains a serialized json object.
            @return dict
        """
        fb_data = getattr(self, 'facebook_data', {})
        if facebook_data:
            fb_data.update(facebook_data)
        raw_app_data = fb_data.get('signed_request', {}).get('app_data', None) or request.REQUEST.get('app_data', None)
        if raw_app_data:
            try:
                app_data = json.loads(self.base64_url_decode(raw_app_data))
            except Exception, ex:
                logger.exception("Errore nel decoding di app_data: %s" % ex)
                app_data = {}
        else:
            app_data = {}
        return app_data
    
    # NON UTILIZZATO ---> Che si fa?
    def encode_app_data(self, app_data = None, request = None, facebook_data = None):
        request = request or getattr(self, "request", None)
        app_data = app_data or getattr(self, 'app_data', None)
        if not app_data:
            if request:
                app_data = self.parse_app_data(request=request, facebook_data=facebook_data)
            else:
                app_data = {}
        encoded =  base64.urlsafe_b64encode(json.dumps(app_data))
        return encoded
        
    def load_facebook_data(self, request, *args, **kwargs):
        """ fetch the facebook 'signed request', then parses it and injects 
            a new attribute 'facebook_data' into the object.
            It also caches the data in a session parameter.
            @set ivar facebook_data
            @set ivar app_data
            @set ivar fb_client
            @set request.session.facebook_data
            @return void
        """
        if request.REQUEST.has_key('signed_request'):
            signed_request = request.REQUEST['signed_request']
            facebook_data = self.parse_signed_request(signed_request)
            request.session['facebook_data'] = facebook_data
        else:
            facebook_data = request.session.get('facebook_data', {})
        self.facebook_data = facebook_data
        self.app_data = self.parse_app_data(request, facebook_data)
        access_token = facebook_data.get('access_token', None)
        logger.info("access_token : %s [%s]" %(access_token, facebook_data))
        if access_token:
            self.fb_client = FBClient(access_token)
        else:
            self.fb_client = None




class FBBaseMixin(FBAppViewMixin):
    """
    questo mixin deve risultare nel MRO prima della View 
    """
    
    @method_decorator(never_cache)
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        #request.body
        self.load_facebook_data(request)
        redirect_response = self.prehandle_response(request, *args, **kwargs)
        if redirect_response:
            return redirect_response
        return super(FBBaseMixin, self).dispatch(request, *args, **kwargs)
    
    def prefetch_data(self, request, *args, **kwargs):
        """ set some object attributes usable by all the handlers 
            @return: void
        """
        pass
    
    def prehandle_response(self, request, *args, **kwargs):
        """ pre-handle actions before calling a request method
            @return: HttpResponse || None
        """
        prefetch_response = self.prefetch_data(request, *args, **kwargs)
        if prefetch_response:
            return prefetch_response
        return None



