# -*- coding: utf-8 -*-
""" fHCoreLibs.apps.facebook.views.game by Frankhood
    @author: Frankhood Business Solutions
    @author: Gaetano D'Onghia
"""
from __future__ import absolute_import, print_function, unicode_literals

from django.contrib import messages
from django.utils.encoding import smart_str
from django.views.generic.base import TemplateResponseMixin, View

from fhcore.apps.facebook.views.exceptions import ActionNotAllowed,\
    NoUserException
from fhcore.apps.facebook.views.mixins import FBContextMixin, FBUserMixin,\
    FBPlayerMixin
from fhcore.views.base import HeadResponseMixin, JSONResponseMixin

import logging
from fhcore.apps.facebook.views.base import BaseFBView, FBBaseMixin
from views.generic.base import TemplateView
from utils.decorators import method_decorator
from views.decorators.csrf import csrf_exempt
from views.decorators.cache import never_cache
from http.response import HttpResponse
logger = logging.getLogger('project')

"""

class BaseFBView(FBBaseMixin, HeadResponseMixin, View):

    @method_decorator(never_cache)
    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        #request.body
        self.load_facebook_data(request)
        rsp = self.prehandle_response(request, *args, **kwargs)
        if rsp:
            return rsp
        try:
            self.prefetch_data(request, *args, **kwargs)
        except NoUserException, ex:
            logger.error(ex)
            pass
        return super(BaseFBView, self).dispatch(request, *args, **kwargs)
    
    def prefetch_data(self, request, *args, **kwargs):
        ''' set some object attributes usable by all the handlers 
            @return: void
        '''
        pass
    
    def prehandle_response(self, request, *args, **kwargs):
        ''' pre-handle actions before calling a request method
            @return: HttpResponse || None
        '''
        return False
    
    def post(self, request, *args, **kwargs):
        return self.get(request, *args, **kwargs) 

class AnonymousView(FBContextMixin, BaseFBView, TemplateView):  
    facebook_id_attribute= "id_attribute"

            
class AuthenticatedBaseUserView(BaseFBView):
    user_model = None

    def prefetch_data(self, request, *args, **kwargs):
        if not hasattr(self, 'fb_user'):
            self.fb_user = self.get_userdata_from_facebook(request)
        logger.debug("passo da AuthenticatedBaseUserView ")
            
    def prehandle_response(self, request, *args, **kwargs):
        self.prefetch_data(request, *args, **kwargs)
        logger.debug("passo da AuthenticatedBaseUserView ")
        logger.debug("self.fb_user : {0} ".format(self.fb_user))
        logger.debug("self.is_token_expired() : {0} ".format(self.is_token_expired()))
        if not self.fb_user or self.is_token_expired():
            
            # Controllo se hanno rifiutato i permessi
            #print request.REQUEST
            login_template = self.get_login_template(request, *args, **kwargs)
            return HttpResponse(login_template)
    
    def is_token_expired(self):
        try:
            # Faccio la chiamata + semplice del mondo per provare se il token e' scaduto
            self.fb_client.get('/me')
            return False
        except:
            return True
"""

class FBUserView(TemplateResponseMixin, FBContextMixin, FBUserMixin, HeadResponseMixin, View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)
    
    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)


class FBAnonymousUserView(FBUserView):
    is_oauth_required = False

class FBAuthenticatedUserView(FBUserView):
    is_oauth_required = True


####PLAYER REQUIRED
class FBPlayerView(TemplateResponseMixin, FBContextMixin, FBPlayerMixin, HeadResponseMixin, View):

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)
    
    def post(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context)
    
    





# DA RIVEDERE
class BaseAPIView(JSONResponseMixin, FBUserView):
    """ 
    a generic (kinda :-DD) REST view 
    non ancora utilizzata
    """
    
    model = None
    add_form = None
    slug_name = 'pk'
        
    def get_slug_value(self):
        val = self.kwargs.get(self.slug_name, None) \
             or self.request.GET.get(self.slug_name, None)
        return val
    
    def get_object(self):
        try:
            obj_lookup = { self.slug_name: self.get_slug_value() }
            obj = self.model.objects.select_related().get(**obj_lookup)
        except self.model.DoesNotExist, ex:
            logger.error(ex)
            obj = None
        return obj
    
    def get_context_data(self, status, msg='', obj=None, **kwargs):
        """ @return <dict> context
            {
                'status': <string> ('ERROR' | 'WARNING' | 'SUCCESS')
                'message': <string>
                [, 'obj_name': <dict> ]
                [, **kwargs]
            }
        """
        context = {}
        context['status'] = messages.DEFAULT_TAGS[status]
        context['message'] = smart_str(msg)
        if obj:
            obj_name = self.model._meta.object_name.lower()
            try:
                context[obj_name] = obj.get_as_dict()
            except Exception, ex:
                logger.warning("Unable to convert object to dict: %s" % ex)
                context[obj_name] = obj
        if len(kwargs):
            context.update(kwargs)
        return context
    
    # -- actions
    
    def _save_data(self):
        raise NotImplementedError
    
    def _fetch_data(self):
        try:
            if not self.object:
                raise ActionNotAllowed(_(u"Attenzione! la risorsa richiesta non esiste"
                                          " o non hai i permessi per accedervi"))
            status = messages.SUCCESS
            context_data = self.get_context_data(status, obj=self.object)
        except ActionNotAllowed, ex:
            status = messages.WARNING
            context_data = self.get_context_data(status, ex)
            logger.warning(ex)
        return context_data
    
    # -- request methods
    
    def get(self, request, *args, **kwargs):
        """ retrieve an object """
        self.object = self.get_object()
        context = self._fetch_data()
        return self.render_to_response(context)
    
    def post(self, request, *args, **kwargs):
        """ save a new object """
        self.object = self.get_object()
        context = self._save_data()
        return self.render_to_response(context)
