# -*- coding: utf-8 -*-
""" SEO MODULE > MODELS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
"""
from django.db import models
from django.db.models.base import get_model
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned
from django.contrib.contenttypes.models import ContentType
from django.contrib.contenttypes import generic
from django.contrib.contenttypes.generic import GenericRelation
from django.utils.translation import ugettext_lazy as _

from fhcore.models.base import BaseModel


class SEODataBase(BaseModel):
    
    page_title = models.CharField(_("page title"), max_length=255, blank=True) 
    description = models.TextField(_("page description"), blank=True)
    keywords = models.TextField(_("keywords"), blank=True)
    
    class Meta(BaseModel.Meta):
        abstract = True
        verbose_name = _("SEO Metadata")
        verbose_name_plural = verbose_name
        

class SEODataHook(SEODataBase):
    
    content_type = models.ForeignKey(ContentType)
    object_id = models.PositiveIntegerField()
    content_object = generic.GenericForeignKey('content_type', 'object_id')
    
    class Meta(SEODataBase.Meta):
        unique_together = (('object_id', 'content_type'),)

    def __unicode__(self):
        return u"SEO meta for %s" % self.content_object


class SEOData(SEODataBase):
    
    related_id = models.SlugField(_("related id"), max_length=64, primary_key=True)

    def __unicode__(self):
        return "SEO data for related id:%s" % self.related_id


class SEODataHandlerMixin(object):
    """ sperimentale! non usare a produzione """
    
    def __new__(cls, *args, **kwargs):
        kls = super(SEODataHandlerMixin, cls).__new__(cls, *args, **kwargs)
        #kls = cls._default_manager.model.__new__(cls, *args, **kwargs)
        seo_data_meta = kls.seo_data.rel.to._meta
        for fname in seo_data_meta.get_all_field_names():
            setattr(kls, 'get_seo_%s'%fname, seo_data_meta.get_field(fname))
        kls = super(SEODataHandlerMixin, cls).__new__(cls, *args, **kwargs)
        return kls
    
    def _fetch_seo_data(self, attr):
        """ cache handler """
        if not hasattr(self, '__seo_data_cache'):
            try:
                self.__seo_data_cache = self.seo_data.get()
            except (ObjectDoesNotExist, MultipleObjectsReturned), ex:
                return None
        option = getattr(self.__seo_data_cache, attr, None)
        return option() if callable(option) else option
