# -*- coding: utf-8 -*-
""" SEO MODULE > TEMPLATE TAGS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
"""
from django import template
from django.template.loader import render_to_string
from django.core.exceptions import ObjectDoesNotExist, MultipleObjectsReturned

from classytags.core import Options
from classytags.core import Tag as TemplateTag
from classytags.arguments import Argument

from ..models import SEOData

register = template.Library()

class GetSEOMetadata(TemplateTag):
    
    name = 'get_seo_metadata'
    default_varname = 'seo_data'
    template = 'metatags.html'
    model = SEOData
    
    options = Options(
        'related_to', Argument('related_to', required=False, resolve=False),
        'for_token', Argument('for_token', required=False, resolve=True),
        'for', Argument('for_obj', required=False, resolve=True),
        'as', Argument('varname', required=False, resolve=False),
    )

    def render_tag(self, context, varname, related_to=None, for_obj=None, for_token=None):
        try:
            if for_obj and hasattr(for_obj, 'seo_data'):
                seo_data = for_obj.seo_data.get()
            elif for_token:
                try:
                    seo_data = SEOData.objects.get(related_id__iexact=for_token)
                except Exception, ex:
                    seo_data = None
            elif related_to:
                seo_data = SEOData.objects.get(related_id__iexact=related_to)
        except (ObjectDoesNotExist, MultipleObjectsReturned), ex:
            seo_data = None
        if varname:
            context[varname or self.default_varname] = locals().get('seo_data', None)
        else:
            varname = self.default_varname
            return render_to_string(self.template, {self.default_varname:locals().get('seo_data', None)})
        return ''

register.tag(GetSEOMetadata)