# -*- coding: utf-8 -*-
""" fhcore.mail_utils.py   by Frankhood
    @author: Frankhood Business Solutions
"""


"""
Abstract: Utils to send html mails
"""
from django.conf import settings
from django.db.models import get_model
from django.template.loader import render_to_string
from django.core.mail import EmailMessage, get_connection

EMAIL_HOST_ALT = settings.EMAIL_HOST
DEFAULT_FROM_EMAIL = getattr(settings, 'DEFAULT_FROM_EMAIL', 'info@frankhood.it')
DEFAULT_MAILERS = [{'mail':DEFAULT_FROM_EMAIL,
                    'username':'',
                    'password':''}] # only one smtp server but thinked for a list of servers

class EmailHtmlMessage(EmailMessage): 
    content_subtype = 'html'

def send_html_mass_mail(datatuple, fail_silently):
    
    # creo un numero di recipients pari al numero di mailers
    mail_recipients = [[] for m in DEFAULT_MAILERS]
    num_rcp = mail_recipients.__len__()
    total_number_of_mails = 0
    for i, data in enumerate(datatuple):
        mail_subject = data[0]
        mail_body = data[1]
        mail_from = data[2]
        recipient_list = data[3]
        numb_of_mails = recipient_list.__len__()
        if numb_of_mails:
            assigned_recipient = i%num_rcp
            m = EmailMessage()
            m.from_email = DEFAULT_MAILERS[assigned_recipient]['mail']
            m.to = recipient_list
            m.subject = mail_subject
            m.body = mail_body
            m.content_subtype = 'html'
            m.extra_headers = {'Reply-To': settings.DEFAULT_FROM_EMAIL}
            m.extra_headers.update({'Disposition-Notification-To':settings.DEFAULT_FROM_EMAIL})
            mail_recipients[assigned_recipient].append(m)
        total_number_of_mails += numb_of_mails
        
    # spedisci tutte le mails
    if total_number_of_mails > 10:
        for i,r in enumerate(DEFAULT_MAILERS):
            current_recipient = mail_recipients[i]
            connection_params = {'host': EMAIL_HOST_ALT, 'fail_silently': fail_silently}
            if len(current_recipient):
                if r['username']:
                    connection_params.update({'username':r['username']})
                if r['password']:
                    connection_params.update({'password':r['password']})
                connection = get_connection(**connection_params)
                n_msgs = connection.send_messages(current_recipient)
                #connection.close()
    else:
        # Se il numero di mail da spedire è ragionevolmente basso,
        # viene usato l'smtp standard.
        chained_recipient = []
        for i in mail_recipients:
            chained_recipient.extend(i)
        connection = get_connection(fail_silently=fail_silently)
        n_msgs = connection.send_messages(chained_recipient)
    return n_msgs



def send_html_mail(subject, message, from_email, recipient_list,
              fail_silently=False, auth_user=None, auth_password=None,
              connection=None):
    """
    Easy wrapper for sending a single message to a recipient list. All members
    of the recipient list will see the other recipients in the 'To' field.

    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.

    Note: The API for this method is frozen. New code wanting to extend the
    functionality should use the EmailHtmlMessage class directly.
    """
    
    connection = connection or get_connection(username=auth_user,
                                    password=auth_password,
                                    fail_silently=fail_silently)
    return EmailHtmlMessage(subject, message, from_email, recipient_list,
                        connection=connection).send()

def send_advanced_mail(subject, message, from_email, recipient_list, bcc_recipient_list=[],
                  fail_silently=False, auth_user=None, auth_password=None,
                  connection=None, headers={}):
    """
    Easy wrapper for sending a single message to a recipient list. All members
    of the recipient list will see the other recipients in the 'To' field.

    If auth_user is None, the EMAIL_HOST_USER setting is used.
    If auth_password is None, the EMAIL_HOST_PASSWORD setting is used.

    Note: The API for this method is frozen. New code wanting to extend the
    functionality should use the EmailMessage class directly.
    """
    connection = connection or get_connection(username=auth_user,
                                              password=auth_password,
                                              fail_silently=fail_silently)
    return EmailMessage(subject, message, from_email, recipient_list,
                        bcc=bcc_recipient_list,connection=connection, headers=headers).send()