# -*- coding: utf-8 -*-
""" fhcore.templatetags.utils.py by Frankhood
    @author: Frankhood Business Solutions
"""
import re
from django import template
from django.utils.encoding import force_unicode
from django.utils.functional import allow_lazy
from django.utils.html import strip_tags, escape
from django.utils.text import truncate_words
 
register = template.Library()

def autolink(text):    
    url_regexp = re.compile(r"""(?i)\b((?:[a-z][\w-]+:(?:/{1,3}|[a-z0-9%])|www\d{0,3}[.]|([a-z0-9.\-]+[.][a-z]{2,4}/))(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:'".,<>?«»“”‘’]))""")
    starting = 0
    while True:
        has_matched = url_regexp.search(text,starting)
        if has_matched:
            link='<a href=\"%(msg)s\" target=\"_blank\">%(msg)s</a>' % {'msg':has_matched.group(0)}
            text=text[:has_matched.start(0)]+link+text[has_matched.end(0):]
            starting=has_matched.start(0)+len(link)
        else:
            break
    return text    
    

def keeptags(value, tags):
    """
    Strips all [X]HTML tags except the space seperated list of tags 
    from the output.
    
    Usage: keeptags:"strong em ul li"
    """
    tags = [re.escape(tag) for tag in tags.split()]
    tags_re = '(%s)' % '|'.join(tags)
    singletag_re = re.compile(r'<(%s\s*/?)>' % tags_re)
    starttag_re = re.compile(r'<(%s)(\s+[^>]+)>' % tags_re)
    endtag_re = re.compile(r'<(/%s)>' % tags_re)
    value = singletag_re.sub('##~~~\g<1>~~~##', value)
    value = starttag_re.sub('##~~~\g<1>\g<3>~~~##', value)
    value = endtag_re.sub('##~~~\g<1>~~~##', value)
    value = strip_tags(value)
    value = escape(value)
    recreate_re = re.compile('##~~~([^~]+)~~~##')
    value = recreate_re.sub('<\g<1>>', value)
    return value



def truncate_chars(s, num, end_text='...'):
    """Truncates a string after a certain number of words. Takes an optional
    argument of what should be used to notify that the string has been
    truncated, defaulting to ellipsis (...)

    Newlines in the string will be stripped.
    """
    s = force_unicode(s)
    length = int(num)
    chars = s
    if len(chars) > length:
        chars = chars[:length]
        if not chars[-1].endswith(end_text):
            chars.append(end_text)
    return u''.join(chars)
truncate_chars = allow_lazy(truncate_chars, unicode)



def htmlattributes(value, arg):
    '''
    usage: {{ form.your_field|hattrs:"class : something, id: openid_identifier" }}
    '''
    attrs = value.field.widget.attrs
    data = arg.replace(' ', '')   
    kvs = data.split(',') 
    for string in kvs:
        kv = string.split(':')
        attrs[kv[0]] = kv[1]
    rendered = str(value)
    return rendered

def subtract(value, arg):
    return value - arg

register.filter('sub',subtract)
register.filter('hattrs', htmlattributes)
register.filter("autolink", autolink)
register.filter("keeptags", keeptags)