# -*- coding: utf-8 -*-
""" fhcore >  utils.backend.py   by Frankhood
    @author: Frankhood Business Solutions
"""
import os

from south.signals import post_migrate


class UploadPath(object):
    '''
    da utilizzare cosi:
    big_snapshot = models.ImageField(_('Big Snapshot'), upload_to=UploadPath('big_snapshot'), blank=True)
    '''
    def __init__(self, attr_name):
        self.attr_name = attr_name
    
    def __call__(self, instance, filename):
        return self.get_upload_path(instance, filename)
    
    def get_upload_path(self, instance, filename):
        return os.path.join('uploads',
                            instance._meta.app_label.lower(),
                            instance.__class__.__name__.lower(),
                            self.attr_name.lower(), filename)


def get_italian_article_for(object_name, is_determinative=False, is_in_the_singular=True):
    """
       Returns the article for the object_name dthat you pass.
       It implements italian rules for determinating it.
    """
    vowels = [u'a', u'e', u'i', u'o', u'u']
    consonants = [unichr(x) for x in range(97,123)]
    [consonants.remove(z) for z in vowels]
    # vedo l'ultima lettera della prima parola x capire a che parola mi riferisco
    gender = 'f' if (object_name.split(' ')[0][-1]=='a' or (object_name.split(' ')[0][-1]=='e' and object_name.split(' ')[0][-2] in ['r','c'])) else 'm'
    if is_determinative:
        if gender == "f":
            if object_name[0] in vowels:
                return u"l'" if is_in_the_singular else u"le"
            else:
                return u"la" if is_in_the_singular else u"le"
        else:
            if object_name[0] in vowels:
                return u"l'" if is_in_the_singular else u"gli"
            elif object_name[0] == u"z" or (object_name[0] == "s" and object_name[1] in consonants):
                #is used for masculine words beginning with z or s + consonant
                return u"lo" if is_in_the_singular else u"gli"
            else:
                return u"il" if is_in_the_singular else u"i"
    else:
        if gender == "f":
            if is_in_the_singular:
                if object_name[0] in vowels and (object_name[0] == u"i" and object_name[1] in vowels):
                    return u"un'" 
                    #apostrofo davanti a vocale, esclusa la i semiconsonante (come in iena)
                else:
                    return u"una" 
            else:
                return u"alcune"
        elif object_name[0] == u"z" or (object_name[0] == "s" and object_name[1] in consonants):
            #is used for masculine words beginning with z or s + consonant
            return u"uno" if is_in_the_singular else u"alcuni"
        else:
            return u"un" if is_in_the_singular else u"alcuni"


def get_subclasses(classes, level=0):
    """
        Return the list of all subclasses given class (or list of classes) has.
        Inspired by this question:
        http://stackoverflow.com/questions/3862310/how-can-i-find-all-subclasses-of-a-given-class-in-python
    """
    # for convenience, only one class can can be accepted as argument
    # converting to list if this is the case
    if not isinstance(classes, list):
        classes = [classes]

    if level < len(classes):
        classes += classes[level].__subclasses__()
        return get_subclasses(classes, level+1)
    else:
        return classes

def receiver_subclasses(signal, sender, dispatch_uid_prefix, **kwargs):
    """
    A decorator for connecting receivers and all receiver's subclasses to signals. Used by passing in the
    signal and keyword arguments to connect::

        @receiver_subclasses(post_save, sender=MyModel)
        def signal_receiver(sender, **kwargs):
            ...
    """
    import logging
    
    def _decorator(func):
        all_senders = get_subclasses(sender)
        logging.info(all_senders)
        for snd in all_senders:
            signal.connect(func, sender=snd, dispatch_uid=dispatch_uid_prefix+'_'+snd.__name__, **kwargs)
        return func
    return _decorator

def update_contenttypes_names(**kwargs):
    
    from django.contrib.contenttypes.models import ContentType
    from django.utils.functional import Promise

    for c in ContentType.objects.all():
        cl = c.model_class()
        # Promises classes are from translated, mostly django-internal models. ignore them.
        if cl and not isinstance(cl._meta.verbose_name, Promise):
            new_name = cl._meta.verbose_name
            if c.name != new_name:
                print u"Updating ContentType's name: '%s' -> '%s'" % (c.name, new_name)
                c.name = new_name
                c.save()

#post_migrate.connect(update_contenttypes_names, dispatch_uid="update_contenttypes")