# -*- coding: utf-8 -*-
import collections
from xml.sax.saxutils import escape as escapeXML

def js_map(iterable, callback):
    mapped = []
    for i in iterable.iterkeys():
        value = callback(iterable[i],i)
        if not value==None:
            mapped.append(value)
    return mapped

def js_reduce(iterable, callback, initial):
    for i in iterable.iterkeys():
        initial = callback(initial, iterable[i], i)
    return initial

def render_matrix(matrix):
    return "%s,%s,%s,%s,%s,%s" % tuple([matrix[x] for x in ['a', 'b', 'c', 'd', 'e', 'f']])


class toSvg(object):
    
    def tag(self, name, attrs, matrix, content = ""):
        def callback(element, name):
            if name == 'trasform':
                return None
            elif name == 'd':
                element = "".join(["%s%s" % (x[0],",".join([str(y) for y in x[1:]])) for x in element]) if isinstance(element, collections.Iterable) else str(element)
            
            return '%s="%s"' % (name, escapeXML(element))
        attrs = ' '.join(js_map(attrs, callback))
        return '<%(name)s%(matrix)s%(attrs)s>%(content)s</%(name)s>' % \
            {
             "name": name,
             "matrix": (' transform="matrix(%s)" ' % str(matrix).replace(r'^matrix\(|\)$', '')) if matrix else ' ',
             "attrs": str(attrs),
             "content": content,
            }
            
    def extractStyle(self, node):
        
        return {
                "font":{
                        "family": node['attrs']['font'].replace(r'^.*?"(\w+)".*$',r'\1'),
                        "size": node['attrs']['font-size'] if node['attrs'].haskey('font-size') else None,
                        }
               }
    
    def styleToString(self, style):
        # TODO: figure out what is 'normal'
        return 'font: normal normal 10px/normal %(fontfamily)s%(fontsize)s' % \
            {
             "fontfamily" :  str(style['font']['family']),
             "fontsize"   :  '; font-size: %spx' % style['font']['size'] if style['font']['size'] else '',
             }
    
    def computeTSpanDy(self, fontSize, line, lines):
        if not fontSize:
            fontSize = 10
        return fontSize * 4.5 / 13 * ( line - .2 - lines / 2 ) * 3.5
    
    
    def serializer_text(self, node):
        
        style = self.extractStyle(node)
        tags = []
        
        def red(initial, value, name):
            if not name in ['text', 'w', 'h']:
                if name == 'font-size':
                    value = '%spx' % value
                initial['name'] = escapeXML(str(value))
            return initial
        
        def callback(text, iterable, line = None):
            tags.append(self.tag(
                                 'text', 
                                 js_reduce(
                                        node['attrs'], 
                                        red, 
                                        {'style': 'text-anchor: middle; %s;' % self.styleToString(style)}
                                       )
                                ),
                                node['matrix'],
                                self.tag(
                                    'tspan',
                                    { 'dy': self.computeTSpanDy(style['font']['size'], line + 1, len(node['attrs']['text'].split('\n'))) },
                                    None,
                                    text
                                    )
                       )
        js_map(node['attrs']['text'].split('\n'), callback)
        return tags
    
    def serializer_path(self, node):
        initial = {} if ( not node.has_key('matrix') or (node['matrix']['a'] == 1 and node['matrix']['d'] == 1 ))  \
            else { 'transform' : render_matrix(node['matrix']) }
        
        def red(initial, value, name):
            if name == 'path':
                name = 'd'
            initial[name] = value if isinstance(value, collections.Iterable) else str(value)
            return initial
        
        return self.tag(
                        'path', 
                        js_reduce(
                               node['attrs'],
                               red,
                               {},
                               ), 
                        render_matrix(node['matrix']), 
                       )
    def render(self, paper, width, height):
        
        svg = '<svg style="overflow: hidden; position: relative;" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" width="%(width)s" version="1.1" height="%(height)s">' % {'width': width, 'height': height}
        
        for node in paper:  
            #if node['style']['display'] == 'none':
            #    continue
            attrs = ''
            
            handler = getattr(self, 'serializer_%s' % node['type'], None)
            if handler:
                svg = svg + handler(node)
                continue
            
            if node['type']=='image':
                attrs = attrs + ' preserveAspectRatio="none"'
                
            for i in node['attrs'].iterkeys():
                name = i
                
                if i=='src':
                    name = 'xlink:href'
                elif i=='transform':
                    name = ''
                
                if name:
                    attrs = attrs + ' %s="%s"' % (name, escapeXML(str(node['attrs'][i])))
            
            svg = svg + '<%(name)s transform="matrix(%(matrix)s)" %(attrs)s></%(name)s>' % \
                {
                 'name': node['type'],
                 'matrix': render_matrix(node['matrix']).replace(r'^matrix\(|\)$', ''),
                 'attrs': attrs,
                 }
        
        svg = svg + '</svg>'   
        
        return svg  
        
    