# -*- coding: utf-8 -*-
"""  fHCoreLibs.views.localized.py   
    @author: Frankhood Business Solutions
    @date : 21/mar/2013
"""
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.utils.translation import get_language
from django.views.generic.base import View
from django.views.generic.detail import SingleObjectMixin, SingleObjectTemplateResponseMixin

from fhcore.views.generic import CustomManagerMixin


class SingleLocalizedObjectMixin(CustomManagerMixin, SingleObjectMixin):
    language_url_kwarg = 'language'
    language_field = 'lang'
    
    def get_language_field(self):
        return self.language_field    

    def get_local_language(self):
        #lang = self.kwargs.get(self.language_url_kwarg, get_language()) or get_language()
        lang = get_language()
        return lang[:2]
    
    def get_context_data(self, **kwargs):
        context_data = super(SingleLocalizedObjectMixin, self).get_context_data(**kwargs)
        context_data.update({'language':self.lang})
        return context_data

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        By default this requires `self.queryset` and a `pk` or `slug` argument
        in the URLconf, but subclasses can override this to return any object.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Next, try looking up by primary key.
        pk = self.kwargs.get(self.pk_url_kwarg, None)
        slug = self.kwargs.get(self.slug_url_kwarg, None)
        lang = self.lang
        if pk is not None:
            queryset = queryset.filter(pk=pk)

        # Next, try looking up by slug.
        elif slug is not None:
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug})

        # If none of those are defined, it's an error.
        else:
            raise AttributeError(u"Generic detail view %s must be called with "
                                 u"either an object pk or a slug."
                                 % self.__class__.__name__)
        queryset = queryset.filter(**{self.get_language_field():lang})
        try:
            obj = queryset.get()
        except ObjectDoesNotExist:
            raise Http404(_(u"No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
        return obj

class BaseLocalizedDetailView(SingleLocalizedObjectMixin, View):
    
    def get(self, request, *args, **kwargs):
        self.lang = self.get_local_language()
        self.object = self.get_object()
        context = self.get_context_data(object=self.object)
        return self.render_to_response(context)

class SingleLocalizedObjectTemplateResponseMixin(SingleObjectTemplateResponseMixin):
    
    def get_context_data(self, **kwargs):
        context_data = super(SingleLocalizedObjectMixin, self).get_context_data(**kwargs)
        context_data.update({'language':self.lang})
        return context_data

class LocalizedDetailView(SingleLocalizedObjectTemplateResponseMixin, BaseLocalizedDetailView):
    """
    Render a "LOCALE detail" view of an object.

    By default this is a model instance looked up from `self.queryset`, but the
    view will support display of *any* object by overriding `self.get_object()`.
    """