# -*- coding: utf-8 -*-
""" fHCoreLibs.views.mails.py   by Frankhood
    @author: Frankhood Business Solutions
"""
from django.conf import settings
from django.contrib.sites.models import Site
from django.core.mail import send_mail
from django.core.urlresolvers import reverse
from django.http import HttpResponseRedirect
from django.template.loader import render_to_string
from django.utils.decorators import method_decorator
from django.views.decorators.cache import never_cache
from django.views.generic.edit import FormView, CreateView, BaseCreateView

from fhcore.exceptions import RedirectException
from fhcore.mail_utils import send_html_mail

DEFAULT_FROM_EMAIL = getattr(settings, 'DEFAULT_FROM_EMAIL', 'info@frankhood.it')
DEFAULT_TO_EMAIL = getattr(settings, 'DEFAULT_FROM_EMAIL', 'info@frankhood.it')

class SendMailMixinView(object):
    '''
    to be called somewhere and needs:
    - context_data 
    - mail_from
    '''
    mail_msg_template = '' #required
    mail_sbj_template = '' #required
    is_html = False
    
    def get_mail_subject(self, context_data):
        return render_to_string(self.mail_sbj_template, context_data)
    
    def get_mail_body(self, context_data):
        return render_to_string(self.mail_msg_template, context_data)
    
    def get_recipient_list(self):
        return [DEFAULT_TO_EMAIL]
    
    def get_email_from(self, mail_from):
        return mail_from
    
    def send_emails(self, context_data, mail_from):
        mail_subject = self.get_mail_subject(context_data)
        mail_body = self.get_mail_body(context_data)
        custom_mail_from = self.get_email_from(mail_from)
        recipient_list = self.get_recipient_list()
        if not self.is_html:
            m = send_mail(mail_subject, mail_body, custom_mail_from, recipient_list, 
                      fail_silently=not settings.DEBUG)
        else:
            m = send_html_mail(mail_subject, mail_body, custom_mail_from, recipient_list, 
                      fail_silently=not settings.DEBUG)
            
   
class SendMailCreateView(SendMailMixinView, CreateView):

    @method_decorator(never_cache)
    def dispatch(self, *args, **kwargs):
        return super(SendMailCreateView, self).dispatch(*args, **kwargs)

    def set_request_data(self, request):
        self._request_data = request.GET.copy()  #qui dentro ho il dizionario con i parametri della query
        self._request_data.update(self.kwargs.copy())
    
    def pre_process_view(self):
        pass
    
    def post_process_view(self, *args, **kwargs):
        pass
    
    def get_object_email(self):
        return ''
    
    def form_valid(self, form):
        self.object = form.save()
        self.send_emails(self.get_context_data(form=form), self.get_object_email())
        return HttpResponseRedirect(self.get_success_url())
    
    def get(self, request, *args, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return super(SendMailCreateView, self).get(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        self.set_request_data(request)
        try:
            self.pre_process_view()
        except RedirectException,e:
            return HttpResponseRedirect(reverse(e.reverse_url, kwargs=e.kw)+ e.querystring)
        return super(SendMailCreateView, self).post(request, *args, **kwargs)
    
    def get_context_data(self, **kwargs):
        context = super(SendMailCreateView, self).get_context_data(**kwargs)
        try:
            current_site = Site.objects.get_current()
        except Exception, ex:
            current_site = None
        context.update({'current_site':current_site})
        if self.object:
            link_dict = {'app_label':self.object._meta.app_label,
                         'module_name': self.object._meta.module_name}
            object_admin_url =reverse('admin:%(app_label)s_%(module_name)s_change' % link_dict,  args=[self.object.id] )
            context.update({'object_admin_url':object_admin_url})
        return context
    

