# -*- coding: utf-8 -*-
"""  frankhood.fhcore.views.response_mixins.py   
    @author: Frankhood Business Solutions
    @date : 20/set/2013
"""
import csv
from django.http import HttpResponse
from django.views.generic.list import BaseListView
from django.conf import settings
import os

class TXTResponseMixin(object):
    
    file_name = None
    file_mimetype  = 'text/plain'
    
    def get_file_name(self):
        return self.file_name
    
    def get_file_mimetype(self):
        return self.file_mimetype
    
    def build_response(self, response, context):
        for row in context['rows']:
            response.write("%s\n" % row)
        return response
        
    def render_to_response(self, context, **response_kwargs):
        """
        Genera una risposta nel mimetype selezionato.
        """
        file_name = self.get_file_name()
        file_mimetype = self.get_file_mimetype()
        if not file_name:
            raise Exception("Manca il nome del file")
        response = HttpResponse(mimetype=file_mimetype)
        response['Content-Disposition'] = 'attachment; filename="%s"' % file_name
        response = self.build_response(response, context)
        return response

class PDFResponseMixin(object):
    
    file_name = None
    file_mimetype = "application/pdf"

    def get_file_name(self):
        return self.file_name
   
    def get_file_dir(self):
        #modifica se non è questo il path del file
        return os.path.join(settings.STATIC_ROOT, 'uploads', 'pdf')
    
    def render_to_response(self, context, **response_kwargs):
        file_path = "{0}{1}{2}".format(self.get_file_dir(),os.sep,self.get_file_name())
        with open(file_path, 'r') as pdf:
            response = HttpResponse(pdf.read(), mimetype=self.file_mimetype)
            response['Content-Disposition'] = 'inline;filename={0}'.format(self.get_file_name())
            return response
        pdf.closed   

class CSVResponseMixin(TXTResponseMixin):
    
    csv_delimiter = ";"
    file_mimetype  = 'text/csv'

    def build_response(self, response, context):
        writer = csv.writer(response,delimiter=self.csv_delimiter)#, quoting=csv.QUOTE_NONNUMERIC)
        for row in context['rows']:
            writer.writerow([s.encode("utf-8") if isinstance(s, basestring) else s for s in row ])
        return response


class CSVListView(CSVResponseMixin, BaseListView):
    
    def get_fields(self):
        return [field for field in self.model._meta.fields]
    
    def get_context_data(self, object_list):
        context = {'rows': []}
        fields = self.get_fields()
        context['rows'].append([unicode(field.verbose_name) if hasattr(field,'verbose_name') else unicode(field) for field in fields])
        for o in object_list:
            try:
                context['rows'].append([getattr(o,field.name) if hasattr(field,'name') else getattr(o,field) for field in fields])
            except Exception, ex:
                if settings.DEBUG:
                    raise ex
        return context