# -*- coding: utf-8 -*-
"""  fHCoreLibs.views.crud.py   
    @author: Frankhood Business Solutions
    @date : 01/lug/2013
"""

import datetime

from django.core import serializers
from django.db.models.query import QuerySet
from django.core.exceptions import ObjectDoesNotExist
from django.http import Http404
from django.utils.translation import ugettext as _
from django.views.generic.detail import DetailView, BaseDetailView
from django.views.generic.list import ListView

try:
    import json
except ImportError:
    from django.utils import simplejson as json

from fhcore.views.base import JSONResponseMixin

############################################################################
########## aSyNCHRONOUS mixin-VIEWS ########################################
############################################################################

class JSONSerializedResponseMixin(object):    
    """
    In pratica la si affianca ad una qualunque vista di Django
    l'unica cosa da fare dovrebbe essere dichiarare quali campi del modello e quali funzioni utilizzare 
    poter richiamare da frontend, tramite update del Context da "get_context_data"
    Esempio :
    
    def get_context_data(self, **kwargs):
        context_data = super(MyListOrDetailView, self).get_context_data(**kwargs) or {}
        context_data.update({
                             'fields':('title', 'slug', 'date_published', 'order'),
                             'extras':('thumbnail_cover_big', 'thumbnail_cover_small',
                                       'thumbnail_placeholder_on', 'thumbnail_placeholder_off',
                                       'get_absolute_url', 'partial_url' ,
                                       'next', 'previous',
                                       'seen', 'publish_date'
                                       ),
                             'extras_context': {'request': self.request},
                             })
        return context_data
        
    """
    
    
    def get_array_object(self):
        ''' 
        you have to insert your object or queryset to be serialized from 
        wadofstuff serializer
        
        example:
             return self.object   or 
             return self.object_list
        '''
        raise NotImplementedError(_("Insert object o object lst to be serialized in get_array_dict method"))

    def add_meta_2_context(self):
        """
        da utilizzare in questa maniera nelle classi derivate
        context_meta = super(YourClass, self).add_meta_2_context(**kwargs) or {}
        context_meta.update(yourDict)
        return context_meta
        """
        context_meta = {}
        return context_meta

    def get_context_data(self, **kwargs):
        context_data = super(JSONSerializedResponseMixin, self).get_context_data(**kwargs) or {}
        if not context_data.has_key('meta'):
            context_data['meta']={}
        context_data['meta'].update(self.add_meta_2_context())
        return context_data
    
    def fashion_data_like_tastypie(self, data):
        """
        Adatta il contenuto e lo rende uguale all'output uscente con tastypie 
        il .js backbone_tastypie.js si aspetta data renderizzati inn questa maniera:
        
        {
        "objects":[{'title':'my_title', ...},{'title':'my_title2', ...}],
        "meta":[{'total_count':5, ...}]
        }
        """
        _objects = []
        for single_object in data:
            removing_items=[]
            obj_dict = single_object['fields']
            if single_object.has_key('extras'):
                for key,value in single_object['extras'].iteritems():
                    obj_dict[key]=value
            #start parte totalmente custom
            #@todo customizzarla ma come? noltre serve solo x una OneToOne o x una ManytoOne
            # se è un dizionario (e non è roba tipo exclude ma nn dovrebbero esserci problemi a riguardo)
            for key, values in single_object['fields'].items():
                if type(values)==dict:
                    #siamo in presenza di una ForeignKey xkè il serializzatore trasforma FK in dict
                    fk_dict= single_object['fields'][key]
                    for fk_key,fk_value in fk_dict['fields'].iteritems():
                        obj_dict["%s__%s" % (key, fk_key)]=fk_value
                    if fk_dict.has_key('extras'):
                        for fk_extra_key,fk_extra_value in fk_dict['extras'].iteritems():
                            obj_dict["%s__%s" % (key, fk_extra_key)]=fk_extra_value
                    removing_items.append(key)
            for removing_key in removing_items:
                if obj_dict.has_key(removing_key):
                    obj_dict.pop(removing_key)
            #end parte totalmente custom
            obj_dict['pk'] = single_object['pk']
            _objects.append(obj_dict)
        return _objects 
    
    def convert_context_to_json(self, context):
        """
        utilizzo di wadofstuff serializer (linka x capirne il meccanismo)
        http://code.google.com/p/wadofstuff/wiki/DjangoFullSerializers
        """
        try:
            """
            bisogna inserire nel context la lista dei fields che si vogliono avere a frontend se si vuole solo un subset
            per fare ciò in get_context_data bisogna inserire qualcosa tipo
            context.update({'fields':['slug', 'created']})
            """
            fields = context['fields'] or None
        except KeyError:
            fields = None
        try:
            """
            bisogna inserire nel context la lista dei fields che non si vogliono esporre 
            per fare ciò in get_context_data bisogna inserire qualcosa tipo
            context.update({'excludes':['modified', 'created']})
            """
            excludes = context['excludes'] or ()
        except KeyError:
            excludes = ()
        try:
            """
            bisogna inserire nel context la lista delle funzioni o property desiderate
            per fare ciò in get_context_data bisogna inserire:
            context.update({'extras':('get_absolute_url', '__unicode__')})
            """
            extras= context['extras'] or {}
        except KeyError:
            extras = ()
        try:
            extras_context= context['extras_context'] or None
        except KeyError:
            extras_context = None
        try:
            """
            bisogna inserire nel context la lista delle relazioni desiderate con se lo si vuole anche extras e fields
            per fare ciò in get_context_data bisogna inserire:
            context.update({'relations':{'user': {'extras':('get_absolute_url', '__unicode__')}}})
            """
            relations= context['relations'] or {}
        except KeyError:
            relations = ()
        try:
            """
            bisogna inserire nel context la lista dei metadata da asociare all'oggetto da serializzare
            per fare ciò in get_context_data bisogna inserire:
            context.update({'meta':{'total_count': 5, 'i_am_winner': True})
            """
            meta= context['meta'] or {}
        except KeyError:
            meta = {}
        output_data = {'meta': meta, 'objects':[]}
        obj_or_objlist = self.get_array_object()
        if obj_or_objlist:
            # Casting a Lista
            if hasattr(obj_or_objlist, '_meta') or type(obj_or_objlist) == QuerySet:
                # obj_or_objlist inherit from:
                    # django.db.models.Model or
                    # django.db.models.query.QuerySet 
                if not type(obj_or_objlist) == QuerySet:
                    # Casting a Lista
                    obj_or_objlist = [obj_or_objlist]
                if fields:
                    json_data = serializers.serialize('json', obj_or_objlist, indent=4, fields=fields, excludes=excludes, extras=extras, relations=relations, extras_context = extras_context)    
                else:
                    json_data =  serializers.serialize('json', obj_or_objlist, indent=4, excludes=excludes, extras=extras, relations=relations , extras_context = extras_context)
                load_data = json.loads(json_data)
                output_data['objects'] = self.fashion_data_like_tastypie(load_data)
            else:
                # obj_or_objlist is a generic list
                output_data['objects'] = obj_or_objlist
        return json.dumps(output_data)

    
class SingleObjectJSONResponseMixin(JSONSerializedResponseMixin, JSONResponseMixin):
    
    pk_url_kwarg ="pk"
    slug_url_kwarg = "slug"
    allow_get_by_pk = True
    allow_get_by_slug = True

    def get_array_object(self):
        return self.object

    def get_object(self, queryset=None):
        """
        Returns the object the view is displaying.

        By default this requires `self.queryset` and a `pk` or `slug` argument
        in the URLconf, but subclasses can override this to return any object.
        """
        # Use a custom queryset if provided; this is required for subclasses
        # like DateDetailView
        if queryset is None:
            queryset = self.get_queryset()

        # Next, try looking up by primary key.
        pk = self.kwargs.get(self.pk_url_kwarg, None)
        slug = self.kwargs.get(self.slug_url_kwarg, None)
        if pk is not None and self.allow_get_by_pk:
            queryset = queryset.filter(pk=pk)

        # Next, try looking up by slug.
        elif slug is not None and self.allow_get_by_slug:
            slug_field = self.get_slug_field()
            queryset = queryset.filter(**{slug_field: slug})

        # If none of those are defined, it's an error.
        else:
            raise AttributeError(u"Generic detail view %s must be called with "
                                 u"either an object pk or a slug."
                                 % self.__class__.__name__)

        try:
            obj = queryset.get()
        except ObjectDoesNotExist:
            obj = None
            """
            raise Http404(_(u"No %(verbose_name)s found matching the query") %
                          {'verbose_name': queryset.model._meta.verbose_name})
            """
        return obj

    def add_meta_2_context(self):
        context_meta =  {'total_count' : 1 if self.object else 0}
        return context_meta

class MultipleObjectJSONResponseMixin(JSONSerializedResponseMixin, JSONResponseMixin):
    
    def get_array_object(self):
        return self.object_list
    
    def add_meta_2_context(self):
        try:
            total_count = self.object_list.count()
        except:
            total_count = self.object_list and len(self.object_list) or 0
        context_meta =  {'total_count' : total_count}
        return context_meta   