# -*- coding: utf-8 -*-
""" CORE MODULE > WIDGETS
    @author: Domenico Lamparelli <domenico@frankhood.it> 
    @author: Nicola Mosca <nico@frankhood.it>
    @author: Gaetano D'Onghia <g.donghia@frankhood.it>
"""

from django import forms
from django.conf import settings
from django.utils.safestring import mark_safe

class ColorPickerWidget(forms.TextInput):
    
    class Media:
        css = {
            'all': (
                '%s/fhcore/colorpicker/css/colorpicker.css' % settings.STATIC_URL,
            )
        }
        js = (
            '%s/fhcore/colorpicker/js/colorpicker.js' % settings.STATIC_URL,
            '%s/fhcore/colorpicker/js/eye.js' % settings.STATIC_URL,
            '%s/fhcore/colorpicker/js/utils.js' % settings.STATIC_URL,
            '%s/fhcore/colorpicker/js/layout.js' % settings.STATIC_URL,
        )

    def __init__(self, language=None, attrs=None):
        self.language = language or settings.LANGUAGE_CODE[:2]
        super(ColorPickerWidget, self).__init__(attrs=attrs)

    def render(self, name, value, attrs=None):
        rendered = super(ColorPickerWidget, self).render(name, value, attrs)
        return rendered + mark_safe(
u'''
<div id="colorSelector_%(name)s" class="colorSelector" style="float:left;"><div></div></div>
<script type="text/javascript">
(function ($) {
    $('#id_%(name)s').hide();
    var startcolor = $('#id_%(name)s').val()||'#ff3300';
    $('#colorSelector_%(name)s div').css('backgroundColor', startcolor);
    $('#colorSelector_%(name)s').ColorPicker({
    color: startcolor,
    onShow: function (colpkr) {
        $(colpkr).fadeIn(500);
        return false;
    },
    onHide: function (colpkr) {
        $(colpkr).fadeOut(500);
        return false;
    },
    onChange: function (hsb, hex, rgb) {
        $('#colorSelector_%(name)s div').css('backgroundColor', '#' + hex);
        $('#id_%(name)s').val('#' + hex);
    }
});
})(django.jQuery);
    </script>''' % {"name": name})
