# -*- coding: utf-8 -*-
"""  fHCoreLibs.widgets.sorl_image_widgets.py   
    @author: Frankhood Business Solutions
    @date : 10/apr/2013
"""
import re
from os import path
    
from django.contrib.admin.widgets import AdminFileWidget
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext as _

try:
    from sorl.thumbnail import get_thumbnail 
    
    
    class PreviewImageWidget(AdminFileWidget):
        
        class Media:
            css = {
                'all': ('common/css/admin_form_overrides.css',)
            }
        
        def get_image_html(self, value, thumbnail):
            output_html = '''
                <div class="box img-preview" style="border:2px solid #9AB8D6;float:left;margin-right:5px;">
                    <h4 style="text-align:center;color:#fff;background:#9AB8D6;">%s</h4>
                    <a target="_blank" href="%s"><img src="%s" alt="%s" /></a>
                </div>%s ''' % (_('Preview'), value.url, thumbnail.url, path.split(value.url)[-1], _('Change:'))
            return output_html
        
        def get_image_opts(self):
            return { 
                    'size':(60,60), 
                    'crop': 'center',#'smart'
                   }
        
        def get_image_geometry(self):
            return "60x60"
    
        def render(self, name, value, attrs=None):
            output = []
            self.value = value
            if value and hasattr(value, "url"):
                try:
                    #realname = re.sub(r'^(\w_-+)-(\d+)-', '', name)
                    realname = re.sub(r'.*?-(\d+)-', '', name)
                    image_preview = getattr(value.instance, realname)
                    geometry = self.get_image_geometry()
                    opts = self.get_image_opts()
                    thumbnail = get_thumbnail(image_preview, geometry, **opts)
                    output.append(self.get_image_html(value, thumbnail))
                except IOError:
                    pass
                except Exception, ex:
                    raise
            output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))
        
    class PreviewBigImageWidget(PreviewImageWidget):
        
        def get_image_html(self, value, thumbnail):
            output_html = '''
                <div class="box img-preview" style="float:left;margin-right:5px;">
                    <a target="_blank" href="%s"><img src="%s" alt="%s" /></a>
                </div>%s ''' % (value.url, thumbnail.url, path.split(value.url)[-1], _('Change:'))
            return output_html
        
        def get_image_opts(self):
            return { 
                    'size':(300,220), 
                    'crop': 'center',#'smart'
                   }
        
        def get_image_geometry(self):
            return "300x220"
    
    class PreviewGenericImageWidget(PreviewImageWidget):
        #DA TESTARE
    
        def render(self, name, value, attrs=None):
            output = []
            if value and hasattr(value, "url"):
                try:
                    realname = re.sub(r'^(\w+)-(\d+)-', '', name)
                    realname_splitted = realname.split('content_type-object_id-')
                    img_name=''
                    for item in realname_splitted:
                        img_name+=item
                    image_preview = getattr(value.instance, img_name)
                    opts = self.get_image_opts()
                    thumbnail = get_thumbnail(image_preview).get_thumbnail(opts)
                    output.append(self.get_image_html(value, thumbnail))
                except IOError:
                    pass
                except Exception, ex:
                    raise
            output.append(super(AdminFileWidget, self).render(name, value, attrs))
            return mark_safe(u''.join(output))

except ImportError, ex:
    pass
